﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.DataPreprocessing {
  internal class PreprocessingCloner {

    private readonly IPreprocessingContext context;

    public PreprocessingCloner(IPreprocessingContext context) {
      this.context = context;
    }

    public IItem GenerateAlteredClone(IItem obj) {
      IPreprocessingData data = context.Data;

      Dataset dataset = data.ExportToDataset();
      var variables = new CheckedItemList<StringValue>(data.VariableNames.Select(s => new StringValue(s))).AsReadOnly();

      var cloner = new Cloner();

      cloner.RegisterClonedObject(context.DataAnalysisProblemData.Dataset, dataset);
      cloner.RegisterClonedObject(context.DataAnalysisProblemData.TrainingPartition, data.TrainingPartition);
      cloner.RegisterClonedObject(context.DataAnalysisProblemData.TestPartition, data.TestPartition);
      cloner.RegisterClonedObject(context.DataAnalysisProblemData.InputVariables, variables);
      if (context.Algorithm != null) {
        cloner.RegisterClonedObject(context.Algorithm.Runs, new RunCollection { OptimizerName = context.Algorithm.Name });
      }

      return cloner.Clone(obj);
    }
  }
}
